package com.gx.obe.component.table;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;

import com.swtdesigner.SWTResourceManager;

import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.SWTX;
import de.kupzog.ktable.editors.KTableCellEditorLink;
import de.kupzog.ktable.editors.KTableCellEditorText;
import de.kupzog.ktable.model.CommonKtableCModel;
import de.kupzog.ktable.renderers.FixedWrapCellRenderer;
import de.kupzog.ktable.renderers.LinkCellRenderer;
import de.kupzog.ktable.renderers.TextWrapCellRenderer;

public class CTable<T> extends KTable {
	
	/**
	 * @Description: 换行文本渲染
	 */
	private static final FixedWrapCellRenderer FIXED_RENDERER = new FixedWrapCellRenderer();
	/**
	 * @Description: 换行文本渲染
	 */
	private static final TextWrapCellRenderer TEXT_RENDERER = new TextWrapCellRenderer();
	
	/**
	 * @Description: 固定行数
	 */
	private static final int HEAD_ROW = 1;
	
	/**
	 * @Description: 默认最小列宽
	 */
	private static int COLUMN_MIN_WIDTH = -1;
	/**
	 * @Description: 默认最小列宽
	 */
	private static int COLUMN_WIDTH = -1;
	/**
	 * @Description: 默认最大列宽
	 */
	private static int COLUMN_MAX_WIDTH = -1;
	
	/**
	 * @Description: 默认列对齐方式
	 */
	private static int ALIGN_HORIZONTAL = SWTX.ALIGN_HORIZONTAL_CENTER;
	
	/**
	 * @Description: 固定列
	 */
	private final List<Column> headColumnList;
	/**
	 * @Description: 内容列
	 */
	private final List<Column> contentColumnList;
	
	/**
	 * @Description: 显示数据
	 */
	private final List<T> tList;
	
	/**
	 * @Description: 表格模型
	 */
	private final CommonKtableCModel commonKtableCModel;
	
	public CTable(Composite parent, List<Column> headColumnList, List<Column> contentColumnList, List<T> tList) {
		super(parent, SWTX.AUTO_SCROLL | SWT.FULL_SELECTION | SWTX.EDIT_ON_KEY | SWTX.FILL_WITH_LASTCOL);
		this.headColumnList = headColumnList;
		this.contentColumnList = contentColumnList;
		this.tList = Optional.ofNullable(tList).orElseGet(ArrayList::new);
		this.commonKtableCModel = new CommonKtableCModel(this, HEAD_ROW, headColumnList.size(), this.tList.size(), contentColumnList.size());
		show();
	}
	
	/**
	 * @Description: 显示
	 * @author chenxw
	 */
	private void show() {
		for (int col = 0; col < commonKtableCModel.getColCount(); col++) {
			int minWidth;
			int width;
			int maxWidth;
			if (col < headColumnList.size()) {
				minWidth = getHeadColumn(col).getMinWidth();
				width = getHeadColumn(col).getWidth();
				maxWidth = getHeadColumn(col).getMaxWidth();
			} else {
				minWidth = Optional.of(getContentColumn(col)).map(Column::getMinWidth).orElse(COLUMN_MIN_WIDTH);
				width = Optional.of(getContentColumn(col)).map(Column::getWidth).orElse(COLUMN_WIDTH);
				maxWidth = Optional.of(getContentColumn(col)).map(Column::getMaxWidth).orElse(COLUMN_MAX_WIDTH);
			}
			commonKtableCModel.setColumnMinWidth(col, minWidth);
			commonKtableCModel.setColumnWidth(col, width);
			commonKtableCModel.setColumnMaxWidth(col, maxWidth);
		}
		
		commonKtableCModel.setRowHeightRender(row -> 30);
		commonKtableCModel.setCellBackgroundRender((a, b) -> SWTResourceManager.getColor(SWT.COLOR_WHITE));
		
		commonKtableCModel.setCellContentRender(this::receiveContent);
		commonKtableCModel.setCellRender(this::receiveCellRender);
		commonKtableCModel.setCellEditorRender(this::receiveCellEditor);
		commonKtableCModel.setCellImageRender(this::receiveCellImage);
		commonKtableCModel.setCellAlignmentRender(this::receiveCellAlignment);
		commonKtableCModel.setCellEditContentRender(this::receiveContent);
		
		this.setModel(commonKtableCModel);
	}
	
	/**
	 * @Description: 加载内容
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private String receiveContent(int row, int col) {
		if (row < HEAD_ROW) {
			if (col < headColumnList.size()) {
				return getHeadColumn(col).getName();
			}
			return getContentColumn(col).getName();
		}
		if (col < headColumnList.size()) {
			return getHeadColumn(col).getFunction().apply(row);
		}
		return Optional.ofNullable(getContentColumn(col).getFunction().apply(row)).orElse("");
	}
	
	/**
	 * @Description: 编辑单元格内容
	 * @author chenxw
	 * @param row
	 * @param col
	 * @param content
	 * @return 
	 */
	private void receiveContent(int row, int col, Object content) {
		if (commonKtableCModel.isFixedCell(col, row)) {
			return;
		}
		Column contentColumn = getContentColumn(col);
		if (!(contentColumn instanceof Text)) {
			return;
		}
		@SuppressWarnings("unchecked")
		Text<T> textContentColumn = (Text<T>) contentColumn;
		textContentColumn.action(getT(row), content);
		redraw();
	}
	
	/**
	 * @Description: 加载渲染器
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private KTableCellRenderer receiveCellRender(int row, int col) {
		if (commonKtableCModel.isFixedCell(col, row)) {
			return FIXED_RENDERER;
		}
		Column contentColumn = getContentColumn(col);
		if (!(contentColumn instanceof Link)) {
			return TEXT_RENDERER;
		}
		@SuppressWarnings("unchecked")
		Link<T> linkContentColumn = (Link<T>) contentColumn;
		LinkCellRenderer linkCellRenderer = new LinkCellRenderer();
		linkCellRenderer.setEnable(linkContentColumn.enable(getT(row)));
		return linkCellRenderer;
	}
	
	/**
	 * @Description: 加载编辑器
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private Object receiveCellEditor(int row, int col) {
		if (commonKtableCModel.isFixedCell(col, row)) {
			return null;
		}
		Column contentColumn = getContentColumn(col);
		if (contentColumn instanceof Link) {
			return new KTableCellEditorLink(this, this::receiveAction);
		} else if (contentColumn instanceof Text) {
			@SuppressWarnings("unchecked")
			Text<T> textContentColumn = (Text<T>) contentColumn;
			KTableCellEditorText cellEditorText = new KTableCellEditorText();
			cellEditorText.setTextLimit(textContentColumn.getTextLimit());
			return cellEditorText;
		}
		return null;
	}
	
	/**
	 * @Description: 按钮点击事件
	 * @author chenxw
	 * @param row
	 * @param col
	 */
	private void receiveAction(int row, int col) {
		if (commonKtableCModel.isFixedCell(col, row)) {
			return;
		}
		Column contentColumn = getContentColumn(col);
		if (!(contentColumn instanceof Link)) {
			return;
		}
		@SuppressWarnings("unchecked")
		Link<T> linkContentColumn = (Link<T>) contentColumn;
		linkContentColumn.action(getT(row));
	}
	
	/**
	 * @Description: 加载图片
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private Image receiveCellImage(int row, int col) {
		if (row < HEAD_ROW) {
			return null;
		}
		if (col < headColumnList.size()) {
			return getHeadColumn(col).getImage();
		}
		return getContentColumn(col).getImage();
	}
	
	/**
	 * @Description: 加载单元格对齐方式
	 * @author chenxw
	 * @param row
	 * @param col
	 * @return
	 */
	private int receiveCellAlignment(int row, int col) {
		if (row < HEAD_ROW) {
			return SWTX.ALIGN_VERTICAL_CENTER | SWTX.ALIGN_HORIZONTAL_CENTER;
		}
		int align;
		if (col < headColumnList.size()) {
			align = Optional.of(getHeadColumn(col)).map(Column::getAlign).orElse(ALIGN_HORIZONTAL);
		} else {
			align = Optional.of(getContentColumn(col)).map(Column::getAlign).orElse(ALIGN_HORIZONTAL);
		}
		return SWTX.ALIGN_VERTICAL_CENTER | align;
	}
	
	/**
	 * @Description: 获取固定列
	 * @author chenxw
	 * @param col
	 * @return
	 */
	private Column getHeadColumn(int col) {
		return headColumnList.get(col);
	}
	
	/**
	 * @Description: 获取内容列
	 * @author chenxw
	 * @param col
	 * @return
	 */
	private Column getContentColumn(int col) {
		return contentColumnList.get(col - headColumnList.size());
	}
	
	/**
	 * @Description: 获取T
	 * @author chenxw
	 * @param row
	 * @return
	 */
	public T getT(int row) {
		row -= HEAD_ROW;
		if (tList.size() > row) {
			return tList.get(row);
		}
		return null;
	}
	
}
