package com.gx.obe.component.table;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.component.listener.SelectionListener;
import com.gx.obe.component.utils.CompositeUtils;

import de.kupzog.ktable.KTableCellSelectionAdapter;

/**
 * @Description: 函数式表格
 * @author chenxw
 * @param <T>
 */
public abstract class FTable<T> extends Composite {
	
	/**
	 * @Description:
	 */
	private CTable<T> cTable;
	
	/**
	 * @Description: 获取t函数
	 */
	private final Function<Integer, T> tFun = this::getT;
	
	/**
	 * @Description: 固定列
	 */
	private final List<Column> headColumnList = new ArrayList<>();
	/**
	 * @Description: 内容列
	 */
	private final List<Column> contentColumnList = new ArrayList<>();
	
	private List<SelectionListener<T>> selectionListenerList = new ArrayList<>();
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param tList
	 */
	public FTable(Composite parent, int style) {
		super(parent, style);
		setLayout(new FillLayout(SWT.HORIZONTAL));
		init();
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	protected abstract void init();
	
	/**
	 * @Description: 刷新
	 * @author chenxw
	 * @param tList
	 */
	public void refresh(List<T> tList) {
		CompositeUtils.disposeChildren(this);
		this.cTable = new CTable<>(this, headColumnList, contentColumnList, tList);
		cTable.addCellSelectionListener(new KTableCellSelectionAdapter() {
			public void cellSelected(int col, int row, int statemask) {
				selection(row);
			}
		});
		this.layout();
		this.getParent().layout();
		selection(1);
	}
	
	private void selection(int row) {
		T t = getT(row);
		if (t != null) selectionListenerList.forEach(l -> l.selection(t));
	}
	
	public void addSelectionListener(SelectionListener<T> selectionListener) {
		this.selectionListenerList.add(selectionListener);
	}
	
	/**
	 * @Description: 获取T
	 * @author chenxw
	 * @param row
	 * @return
	 */
	private T getT(int row) {
		return this.cTable.getT(row);
	}
	
	/**
	 * @Description: 索引列
	 * @author chenxw
	 */
	protected class IndexColumn extends Column {
		public IndexColumn(String name) {
			this(name, 50, String::valueOf);
		}
		
		public IndexColumn(String name, int width, Function<Integer, String> function) {
			super(name, function);
			headColumnList.add(this);
			width(width).minWidth(width).maxWidth(width);
		}
	}
	
	/**
	 * @Description: 固定列
	 * @author chenxw
	 */
	protected class HeadColumn extends Column {
		public HeadColumn(String name, int width, Function<T, String> function) {
			super(name, tFun.andThen(function));
			headColumnList.add(this);
			width(width).minWidth(width).maxWidth(width);
		}
	}
	
	/**
	 * @Description: 内容列
	 * @author chenxw
	 */
	protected class ContentColumn extends Column {
		public ContentColumn(String name, Function<T, String> function) {
			super(name, tFun.andThen(function));
			contentColumnList.add(this);
		}
	}
	
	/**
	 * @Description: 链接内容列
	 * @author chenxw
	 */
	protected class LinkContentColumn extends ContentColumn implements Link<T> {
		private Consumer<T> consumer;
		private Predicate<T> predicate = t -> true;
		
		public LinkContentColumn(String name, String content, Image image, Consumer<T> consumer) {
			super(name, t -> content);
			image(image);
			this.consumer = consumer;
		}
		
		public void enable(Predicate<T> predicate) {
			this.predicate = Objects.requireNonNull(predicate);
		}
		
		@Override
		public boolean enable(T t) {
			return predicate.test(t);
		}
		
		@Override
		public void action(T t) {
			consumer.accept(t);
		}
	}
	
	/**
	 * @Description: 文本编辑内容列
	 * @author chenxw
	 */
	protected class TextContentColumn<R> extends ContentColumn implements Text<T> {
		
		private Function<Object, R> format;
		private BiConsumer<T, R> biConsumer;
		private int textLimit = 1000;
		
		public TextContentColumn(String name, Function<T, String> function, Function<Object, R> format, BiConsumer<T, R> biConsumer) {
			super(name, function);
			this.format = format;
			this.biConsumer = biConsumer;
		}
		
		public void limit(int textLimit) {
			this.textLimit = textLimit;
		}
		
		@Override
		public void action(T t, Object content) {
			biConsumer.accept(t, format.apply(content));
		}
		
		@Override
		public int getTextLimit() {
			return textLimit;
		}
		
	}
	
}
