package com.gx.obe.component.tree;

import java.util.ArrayList;
import java.util.List;

import com.gx.obe.util.utils.CollectionUtils;

/**
 * @Description: 解析树
 * @author chenxw
 */
public class ParseTrees<T extends ITree<T>> {
	
	/**
	 * @Description: 叶子节点列表
	 */
	private List<Tree<T>> leafList = new ArrayList<>();
	
	/**
	 * @Description: 树横向合并列表
	 */
	private List<TreeSpan> wideSpanList = new ArrayList<>();
	/**
	 * @Description: 树竖向合并列表
	 */
	private List<TreeSpan> highSpanList = new ArrayList<>();
	
	/**
	 * @Description: 高度
	 */
	private int height;
	/**
	 * @Description: 叶子节点个数
	 */
	private int leafCount;
	
	/**
	 * @Description: 空树
	 */
	private final Tree<T> NULL_TREE = new Tree<>();
	
	/**
	 * @author chenxw
	 * @param iTreeList
	 */
	public ParseTrees(List<? extends T> iTreeList) {
		if (CollectionUtils.isNull(iTreeList)) {
			return;
		}
		initLeafList(iTreeList, 0, null);
		initHeight(iTreeList, 1);
		
		initHighSpanList();
		initWideSpanList(iTreeList, 0, 0);
		
		initLeafCount();
	}
	
	/**
	 * @Description: 初始化叶子节点列表
	 * @author chenxw
	 * @param iTreeList
	 * @param level
	 * @param parent
	 */
	private void initLeafList(List<? extends T> iTreeList, int level, Tree<T> parent) {
		for (T iTree : iTreeList) {
			Tree<T> tree = new Tree<>();
			tree.setLevel(level);
			tree.setParent(parent);
			tree.setValue(iTree);
			List<? extends T> iTrees = iTree.getITrees();
			if (CollectionUtils.isNull(iTrees)) leafList.add(tree);
			else initLeafList(iTrees, level + 1, tree);
		}
	}
	
	/**
	 * @Description: 初始化树横向合并列表
	 * @author chenxw
	 * @param iTreeList
	 * @param level
	 * @param leafIndex
	 */
	private int initWideSpanList(List<? extends ITree<T>> iTreeList, int level, int leafIndex) {
		for (ITree<T> iTree : iTreeList) {
			List<? extends ITree<T>> iTrees = iTree.getITrees();
			if (CollectionUtils.isNull(iTrees)) leafIndex++;
			else {
				TreeSpan span = new TreeSpan();
				span.setLevel(level);
				span.setLeafIndex(leafIndex);
				span.setSpanCount(iTrees.size());
				wideSpanList.add(span);
				leafIndex = initWideSpanList(iTree.getITrees(), level + 1, leafIndex);
			}
		}
		return leafIndex;
	}
	
	/**
	 * @Description: 初始化树竖向合并列表
	 * @author chenxw
	 */
	private void initHighSpanList() {
		for (int i = 0; i < leafList.size(); i++) {
			Tree<T> tree = leafList.get(i);
			if (height > tree.getLevel()) {
				TreeSpan span = new TreeSpan();
				span.setLevel(tree.getLevel());
				span.setLeafIndex(i);
				span.setSpanCount(height - tree.getLevel());
				highSpanList.add(span);
			}
		}
	}
	
	/**
	 * @Description: 初始化树的高度
	 * @author chenxw
	 * @param iTreeList
	 * @param level
	 */
	private void initHeight(List<? extends ITree<T>> iTreeList, int level) {
		for (ITree<T> iTree : iTreeList) {
			List<? extends ITree<T>> iTrees = iTree.getITrees();
			if (CollectionUtils.isNull(iTrees)) height = Math.max(height, level);
			else initHeight(iTrees, level + 1);
		}
	}
	
	/**
	 * @Description: 初始化叶子节点的数量
	 * @author chenxw
	 */
	private void initLeafCount() {
		leafCount = leafList.size();
	}
	
	/**
	 * @Description: 根据叶子节点索引获取叶子节点
	 * @author chenxw
	 * @param leafIndex
	 * @return
	 */
	public T get(int leafIndex) {
		return leafList.get(leafIndex).getValue();
	}
	
	/**
	 * @Description: 根据层数及叶子节点索引获取对应包含这个叶子节点是节点
	 * @author chenxw
	 * @param level
	 * @param leafIndex
	 * @return
	 */
	public ITree<T> getITree(int level, int leafIndex) {
		return getTree(level, leafList.get(leafIndex)).getValue();
	}
	
	/**
	 * @Description: 根据层数递归获取节点
	 * @author chenxw
	 * @param level
	 * @param tree
	 * @return
	 */
	private Tree<T> getTree(int level, Tree<T> tree) {
		if (level > tree.getLevel() || level < 0) {
			return NULL_TREE;
		}
		if (level == tree.getLevel()) {
			return tree;
		}
		return getTree(level, tree.getParent());
	}
	
	/**
	 * @Description: 获取树横向合并列表
	 * @author chenxw
	 * @return
	 */
	public List<TreeSpan> getWideSpanList() {
		return wideSpanList;
	}
	
	/**
	 * @Description: 获取树竖向合并列表
	 * @author chenxw
	 * @return
	 */
	public List<TreeSpan> getHighSpanList() {
		return highSpanList;
	}
	
	/**
	 * @Description: 获取数的高度
	 * @author chenxw
	 * @return
	 */
	public int getHeight() {
		return height;
	}
	
	/**
	 * @Description: 获取叶子节点数
	 * @author chenxw
	 * @return
	 */
	public int getLeafCount() {
		return leafCount;
	}
	
}
